import logging
import re

import requests
from bs4 import BeautifulSoup

from hentaila.la_types.errors import LaPageLenError
from hentaila.la_types.types_la import LaHentai, LaInfoH, LaInfo
from . import _base, directory_base
from .simple_utils import _base_func, iter_base_func

logger = logging.getLogger("LaLogger")


def get_dict_directory(info) -> LaInfoH:
    thumb = _base + info.find("img").get("src")
    title = info.find("h2").get_text()
    anime_url = _base + info.find("a").get("href")
    return LaInfoH(title, anime_url, thumb)


def get_dict_recent(info) -> LaInfo:
    thumb = _base + info.find("img").get("src")
    title = info.find("h2").get_text()
    chapter_url = _base + info.find("a").get("href")
    anime_url = chapter_url[:-2]
    episode = int(chapter_url.split("-")[-1])
    return LaInfo(title, anime_url, chapter_url, thumb, episode)


def filters_la(
        page: int = 1,
        _filter: str = "recent",
        status: int = 0,
        other: bool = False) -> dict:
    """
    :param page: number of pages
    :param _filter: "recent" or "popular"
    :param status: set 1 for "emission", set 2 for "finalized".
    :param other: True for uncensored, False for censured
    :return: filters dict
    """
    _filters = {
        "filter": _filter,
        "p": page
    }
    if status:
        _filters[f"status[{status}]"] = "on"
    elif other:
        _filters["uncensored"] = "on"
    _max_page = total_count(_filters)
    if page > _max_page:
        raise LaPageLenError(
            f"Page number {page} is larger than total page number {_max_page}")
    return _filters


def total_count(_filters: dict = None) -> int:
    _filters.pop("p")
    r = requests.get(directory_base, params=_filters)
    soup = BeautifulSoup(r.content, 'html.parser')
    return int(
        soup.find(
            "ul",
            {
                "class": "pagination episodes_nav-list"
            }).find_all("a")[-2].get_text())


class Hentaila:
    @staticmethod
    def directory(page=1, _filter="recent", status=0, other=False) -> list[LaInfoH]:
        """
        :param page: number of pages
        :param _filter: "recent" or "popular"
        :param status: set 1 for "emission", set 2 for "finalized".
        :param other: True for uncensored, False for censured
        :return: list of hentai's
        """
        _filters = filters_la(page, _filter, status, other)
        r = requests.get(
            directory_base,
            params=_filters
        )
        soup = BeautifulSoup(r.content, 'html.parser')
        _directory = soup.find("div", {"class": "grid hentais"}).find_all("article")
        return list(map(get_dict_directory, _directory))

    @staticmethod
    def get_hentai(url) -> LaHentai:
        r = requests.get(url)
        soup = BeautifulSoup(r.content, 'html.parser')
        episodes = soup.find("div", {"class": "episodes-list"}).find_all("a")
        genres = soup.find("nav", {"class": "genres"}).find_all("a")
        chapters = list(map(iter_base_func, episodes))[::-1]
        return LaHentai(
            url,
            chapters,
            soup.find("h1", {"class": "h-title"}).text,
            map(lambda x: (x.text, iter_base_func(x)), genres),
            chapters
        )

    @staticmethod
    def get_chapter(url) -> list:
        r = requests.get(url)
        soup = BeautifulSoup(r.content, 'html.parser')
        __videos = soup.find_all("script")[-3].text
        chapters = soup.find("div", {"class": "download-links"}).find_all("a")
        __chapters_videos = []
        __chapters_videos.extend(
            map(lambda x: x.replace("\\", ""), re.findall(r'https?[a-zA-Z0-9!#.:-\\/-]*', __videos))
        )
        __chapters_videos.extend(map(_base_func, chapters))
        return __chapters_videos

    @staticmethod
    def recents() -> list[LaInfo]:
        r = requests.get(_base)
        soup = BeautifulSoup(r.content, 'html.parser')
        _recents = soup.find("div", {"class": "grid episodes"}).find_all("article")
        return list(map(get_dict_recent, _recents))



class LaURL:
    def __init__(self, url):
        self.url = url
        __url_split = url.split("/")
        self.protocol = __url_split[0][:-1]
        self.domain = __url_split[2]
        self.full_slug = __url_split[-1]
        self.slug = __url_split[-1].replace("hentai-", "")

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self.__dict__)


class LaHentai:
    def __init__(
            self,
            parse_url,
            count,
            title,
            genres,
            chapters
    ):
        self.parse_url = LaURL(parse_url)
        self.count = len(count)
        self.title = title
        self.genres = list(genres)
        self.chapters = chapters

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self.__dict__)


class LaInfoH:
    def __init__(
            self,
            title,
            url,
            thumb
    ):
        self.title = title
        self.url = url
        self.thumb = thumb

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self.__dict__)


class LaInfo(LaInfoH):
    def __init__(
            self,
            title,
            url,
            chapter_url,
            thumb,
            episode
    ):
        super().__init__(title, url, thumb)
        self.chapter_url = chapter_url
        self.episode = episode

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return str(self.__dict__)



import pytest

from hentaila_site.hentaila.hentaila import Hentaila
from hentaila_site.hentaila.la_types.errors import LaPageLenError

h = Hentaila()
_hentai = "https://www3.hentaila.com/hentai-euphoria-"
_chapter = "https://www3.hentaila.com/ver/euphoria--1"


def test_hentaila_directory():
    __directory = h.directory()
    assert isinstance(__directory, list)
    assert len(__directory) > 0
    print()
    print(__directory)


@pytest.mark.xfail(raises=LaPageLenError)
def test_directory_fail_max_page():
    assert len(h.directory(15, other=True)) > 0


def test_get_hentai():
    __hentai = h.get_hentai(_hentai)
    assert len(__hentai.genres) > 0
    assert len(__hentai.chapters) > 1
    assert isinstance(__hentai.title, str)
    print()
    print(__hentai)


def test_get_recents():
    __recents = h.recents()
    assert isinstance(__recents, list)
    print()
    print(__recents)


def test_get_chapter():
    __chapter = h.get_chapter(_chapter)
    assert isinstance(__chapter, list)
    print()
    print(__chapter)



from setuptools import setup, find_packages
from hentaila.__vars__ import __version__, __author__, __email__


setup(
    name='hentaila',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/TaprisSugarbell/hentaila',
    author=__author__,
    author_email=__email__,
    description='Unofficial https://www3.hentaila.com/ Package',
    long_description=open("./README.md", "r").read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "beautifulsoup4",
        "cloudscraper"
    ],
    project_urls={
        "Issue tracker": "https://github.com/TaprisSugarbell/hentaila/issues"
    }
)
